@@ -265,7 +265,7 @@ test_that("new_quosure() checks input", {
 })
 
 test_that("as_string(quo) produces informative error message", {
-  expect_error(as_string(quo(foo)), "a `quosure/formula` object to a string")
+  expect_error(as_string(quo(foo)), "a <quosure> object to a string")
 })
 
 test_that("`[` properly reconstructs quosure lists", {
@@ -279,12 +279,19 @@ test_that("quosure lists are considered vectors", {
   expect_identical(vctrs::vec_slice(quos(1, 2, 3), 2:3), quos(2, 3))
 })
 
+test_that("quosure attributes are cloned (#1142)", {
+  x <- quos()
+  attr(x, "foo") <- TRUE
+  y <- quos()
+  expect_true(setequal(names(attributes(y)), c("names", "class")))
+})
+
 
 # Lifecycle ----------------------------------------------------------
 
 test_that("as_quosure() still provides default env", {
   local_lifecycle_warnings()
-  quo <- expect_warning(as_quosure(quote(foo)), "explicit environment")
+  expect_warning(quo <- as_quosure(quote(foo)), "explicit environment")
   expect_reference(quo_get_env(quo), current_env())
 })
 
